/***************************************************************************
                          prefcolor.cpp  -  description
                             -------------------
    begin                : Fri Jun 13 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlistview.h>
#include <qpainter.h>
#include <kcolordialog.h>
#include "prefcolor.h"
#include "kscopeconfig.h"

/**
 * A list view item that shows the name of a GUI element and the colour
 * associated with it.
 * The colour is presented in the form of a rectangle filled with that
 * colour.
 * @author	Elad Lahav
 */
class ColorListItem : public QListViewItem
{
public:
	/**
	 * Class constructor.
	 * @param	pList	The owner list view
	 * @param	ce		The GUI element shown by this item
	 */
	ColorListItem(QListView* pList, KScopeConfig::ColorElement ce) : 
		QListViewItem(pList, Config().getColorName(ce), ""),
		m_ce(ce) {
		setColor(Config().getColor(ce));
	}
	
	/**
	 * @return	The GUI element shown by this item
	 */
	KScopeConfig::ColorElement getElement() { return m_ce; }
	
	/**
	 * Changes the colour associated with this item.
	 * The function assigns a pixmap to the item which shows a rectangle
	 * filled with the requested colour.
	 * The colour set by this function is returned by getColor().
	 * @param	clr		The colour to set
	 */
	void setColor(QColor clr) {
		QPixmap pix;
		QPainter painter;
		int nWidth, nHeight;
		
		// Remember the colour
		m_clr = clr;
		
		// Set the pixmap's size to fit into the list field
		nWidth = listView()->columnWidth(1) - 1;
		nHeight = height();
		pix.resize(nWidth, nHeight);
		
		// Draw on the pixmap
		painter.begin(&pix);
		painter.setBrush(clr);
		painter.drawRect(0, 0, nWidth, nHeight);
		painter.end();
		
		// Set the pixmap to the item
		setPixmap(1, pix);
	}
	
	/**
	 * @return	The colour associated with this item
	 */
	QColor getColor() { return m_clr; }
	
private:
	/** The GUI element shown by this item. */
	KScopeConfig::ColorElement m_ce;
	
	/** The colour associated with this item. */
	QColor m_clr;
};

/**
 * Class constructor.
 * @param	pParent	The parent widget
 * @param	szName	The widget's name
 */
PrefColor::PrefColor(QWidget* pParent, const char* szName) :
	PrefColorLayout(pParent, szName)
{
	m_pList->setColumnWidthMode(1, QListView::Manual);
	
	// Set initial values
	load();
}

/**
 * Class destructor.
 */
PrefColor::~PrefColor()
{
}

/**
 * Reads the current settings from the configuration object, and applies them
 * the the page's widget.
 */
void PrefColor::load()
{
	uint i;
	ColorListItem* pItem;
	
	// Create a list item for every GUI element
	for (i = 0; i <= KScopeConfig::LAST_COLOR; i++)
		pItem = new ColorListItem(m_pList, (KScopeConfig::ColorElement)i);
}

/**
 * Commits settings changes to the configuration object.
 */
void PrefColor::apply()
{
	ColorListItem* pItem;
	
	// Create a list item for every GUI element
	for (pItem = (ColorListItem*)m_pList->firstChild(); pItem != NULL; 
		pItem = (ColorListItem*)pItem->nextSibling()) {
		Config().setColor(pItem->getElement(), pItem->getColor());
	}
}

/**
 * Displays a colour selection dialogue when an item is selected.
 * If the user chooses a new colour, it is set to the selected item.
 * This slot is connected to both the doubleClicked() and the returnPressed()
 * signals of the list view.
 * @param	pItem	The selected item
 */
void PrefColor::slotItemSelected(QListViewItem* pItem)
{
	ColorListItem* pClrItem;
	QColor clr;
	
	pClrItem = (ColorListItem*)pItem;
	if (KColorDialog::getColor(clr, pClrItem->getColor()) == 
		QDialog::Accepted) {
		pClrItem->setColor(clr);
		emit modified();
	}
}

#include "prefcolor.moc"
